import { MigrationInterface, QueryRunner, TableColumn } from "typeorm";

export class Ownership1728313041449 implements MigrationInterface {
  name = "Ownership1728313041449";

  public async up(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.addColumn(
      "user",
      new TableColumn({
        name: "isOwner",
        type: "tinyint",
        default: 0,
        isNullable: false,
      })
    );

    await queryRunner.manager
      .createQueryBuilder()
      .update("user")
      .set({ isOwner: 1 })
      .where((qb) => {
        const subQuery = queryRunner.manager
          .createQueryBuilder()
          .select("1")
          .from("user_permission", "up")
          .where("user.id = up.userId")
          .andWhere("up.permission = '*'")
          .getQuery();
        return `EXISTS (${subQuery})`;
      })
      .execute();

    await queryRunner.manager.createQueryBuilder().delete().from("user_permission").where("permission = '*'").execute();
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.manager
      .createQueryBuilder()
      .insert()
      .into("user_permission")
      .values(
        await queryRunner.manager
          .createQueryBuilder()
          .select("user.id", "userId")
          .addSelect("'*'", "permission")
          .from("user", "user")
          .where("user.isOwner = 1")
          .execute()
      )
      .execute();

    await queryRunner.manager.createQueryBuilder().update("user").set({ isOwner: 0 }).where("isOwner = 1").execute();

    await queryRunner.dropColumn("user", "isOwner");
  }
}
