import { dataSource } from "../../data-source";
import { apiPermission } from "../../entity/user/api_permission";
import InternalException from "../../exceptions/internalException";

export default abstract class ApiPermissionService {
  /**
   * @description get permission by api
   * @param apiId number
   * @returns {Promise<Array<apiPermission>>}
   */
  static async getByApi(apiId: number): Promise<Array<apiPermission>> {
    return await dataSource
      .getRepository(apiPermission)
      .createQueryBuilder("api_permission")
      .where("api_permission.apiId = :apiId", { apiId: apiId })
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("api permissions not found by api", err);
      });
  }
}
